@@ -1,8 +1,8 @@
 #!/usr/bin/perl
-# DocumentId:	$Id: debarchiver.pl,v 1.40 2003/08/28 09:45:27 ola Exp $
+# DocumentId:	$Id: debarchiver.pl 1971 2005-01-01 19:09:29Z ola $
 # Author:	$Author: ola $
-# Date:		$Date: 2003/08/28 09:45:27 $
-# Version:	$Revision: 1.40 $
+# Date:		$Date: 2005-01-01 20:09:29 +0100 (Sat, 01 Jan 2005) $
+# Version:	$Revision: 1.49 $
 # Summary:
 #	This program reads a config file, traverse through all .changes files
 #	in the specified directory and puts the files into the right place.
@@ -29,6 +29,7 @@ $rmcmd = "rm -f";
 $mailcmd = "mail";
 $movecmd = "mv";
 $vrfycmd = "dscverify";
+$cachedir = "/var/cache/debarchiver";
 $inputdir = "/var/lib/debarchiver/incoming";
 $destdir = "/var/lib/debarchiver/dists";
 $cinstall = "installed";
@@ -54,20 +55,24 @@ $userconfigfile = "$ENV{HOME}/.debarchiver.conf";
 $CChanges = "";
 # Destination directories that should be scanned.
 %dests = ();
+# Information to add to Release files.
+%release = ();
 # The criteria to use for which binary packages that does not need a .changes
 # file.
 $distinputcriteria = "^kernel.*\\.deb\$";
-# Extra directories with specified destination.
+# Extra directories with specified destination. Where to find for distinput
+# files.
 %distinputdirs = (
 		  stable => 'stable',
+		  testing => 'testing',
 		  unstable => 'unstable'
 		 );
 
 @distributions = ('stable', 'testing', 'unstable');
 
 %distmapping = (
-		stable => 'woody',
-		testing => 'sarge',
+		stable => 'sarge',
+		testing => 'etch',
 		unstable => 'sid'
 	       );
 
@@ -157,7 +162,7 @@ if (-e $inputconfigfile) {
 ############################# HELP ############################################
 ###############################################################################
 
-$version = "0.0.35";
+$version = "0.0.38";
 $versionstring = "Debian package archiver, version $version";
 
 $help =
@@ -184,15 +189,23 @@ options:
 			 \$distrib/\$major/\$arch/\$section directory structure
 			 will be created. Default $destdir, relative to the
 			 input directory.
+ --scandetect | -s      Scan using apt-ftparchive or dpkg-scan* depending on
+                         what is installed on the system. This is the
+                         recommended way. Only use --index or --autoscan if
+                         you know what you are doing.
+ --index | -x           Automatically run apt-ftparchive after all new packages
+                         are installed.  Use this *or* --autoscan, not both.
  -i | --input dir      	This is the directory where debarchiver is looking for
   --indir dir		 new packages corresponding *.changes files that
   --inputdir dir	 should be installed to the --dest directory.
 			 The default directory is
 			 /var/lib/debarchiver/incoming.
+ --cachedir dir         The apt-ftparchive package cache directory, if --index
+                         is used.  Default $cachedir.
  --lockfile file	The lockfile to use, default $lockfile.
  --cinstall dir		Where the .changes file will be installed to,
 			 empty string to remove it instead, default $cinstall.
- --distinputcritera     The criteria for which binary packages that should be
+ --distinputcriteria     The criteria for which binary packages that should be
 			 installed even if it does not have a .changes file,
 			 default $distinputcriteria.
  -o | --addoverride    	Automaticly add new packages to the override file.
@@ -201,6 +214,7 @@ options:
  --autoscansources	Automaticly run dpkg-scansources after all new
 			 packages are installed.
  -a | --autoscan       	Does both --autoscanpackages and --autoscansources.
+                         Use this *or* --index, not both.
  --scanall		Scan all distributions, sections, etc.
  --autoscanall		Same as --scanall --autoscan.
  --nosort		Do not sort packages.
@@ -213,21 +227,26 @@ $etcconfigfile, $userconfigfile and $inputconfigfile".
 will be parsed. Here you can change the following variables
 	\$destdir      	The destination directory (see --destdir above).
 	\$inputdir	The input directory (no effect in $inputconfigfile).
+        \$cachedir	The cache directory for apt-ftparchive, used if
+			--index is used.
 	\$copycmd	The install command (see --copycmd).
 	\$movecmd	The move command (see --movecmd).
 	\$rmcmd		The remove command (see --rmcmd above).
 	\$cinstall	Where the .changes files are installed
 			 (see --cinstall above).
-        \$distinputcritera The criteria for which packages that should be
+        \$distinputcriteria The criteria for which packages that should be
 			 installed even if it does not have a .changes file,
 			 default $distinputcriteria.
         \%distinputdirs The directories (distribution => dir) that should be
 			 searched for extra bianry packages that does not need
 			a .changes file to be installed.
 	\$lockfile	The lockfile to use, default $lockfile.
-	@mailtos	The fields in .changes file that should be used for
+	\@mailtos	The fields in .changes file that should be used for
 			mailing SUCCESS and REJECT messages. If there is an
 			@ char in the arrach it will be used directly.
+        \%release	Additional information to add to generated Release
+			files.  Supported keys are origin, label, and
+			description.
 ";
 
 ###############################################################################
@@ -274,6 +293,9 @@ while ($_ = shift @ARGS2) {
       $destdir = shift @ARGS2;
       $destdir =~ s/\/$//;
     }
+    elsif (/^--cachedir$/) {
+      $cachedir = shift @ARGS2;
+    }
     elsif (/^--lockfile$/) {
       $lockfile = shift @ARGS2;
     }
@@ -294,6 +316,22 @@ while ($_ = shift @ARGS2) {
       $autoscanpackages = 1;
       $autoscansources = 1;
     }
+    elsif (/^-x$/ || /^--index$/) {
+      $indexall = 1;
+    }
+    elsif (/^-s$/ || /^--scandetect$/) {
+      if (-x "/usr/bin/apt-ftparchive") {
+	$indexall = 1;
+      }
+      else {
+	if (-x "/usr/bin/dpkg-scansources") {
+	  $autoscansources = 1;  
+	}
+	if (-x "/usr/bin/dpkg-scanpackages") {
+	  $autoscanpackages = 1;  
+	}
+      }
+    }
     elsif (/^--distinputcriteria/) {
       $distinputcriteria = shift @ARGS2;
     }
@@ -351,7 +389,11 @@ if (defined $scanall) {
 action(! chdir $inputdir, "Change to dir $inputdir", 2);
 handleSorting();
 
-handleScan();
+if (defined $indexall) {
+  handleIndex();
+} else {
+  handleScan();
+}
 
 ###############################################################################
 ########################### FUNCTIONS #########################################
@@ -384,6 +426,61 @@ sub createPF($$) {
   }
 }
 
+###############################################################################
+# Name:		createRelease
+# Description:	Create a Release file.
+# Dates:	2004-08-09	Written.
+#               2004-08-10      Renamed $release to $contents.
+#                               Don't generate if any under a symlink.
+# Arguments:	directory, distribution, section, architecture
+# Uses:         %release
+###############################################################################
+
+sub createRelease($$$$) {
+  my ($dir, $distribution, $section, $architecture) = @_;
+  my $contents = '';
+  $contents .= "Archive: $distribution\n";
+  $contents .= "Component: $section\n";
+  $contents .= "Label: $release{label}\n" if defined $release{label};
+  $contents .= "Origin: $release{origin}\n" if defined $release{origin};
+  $contents .= "Architecture: $architecture\n";
+  $contents .= "Description: $release{description}\n"
+      if defined $release{description};
+  $contents .= "\n";
+
+  # Don't generate a Release file if any level of the directory is a symlink,
+  # since otherwise for a testing -> unstable symlink, we'll keep regenerating
+  # the Release file, first for testing and then for unstable.  Assume that
+  # we'll also be called with the non-symlink path and create the Release file
+  # then.
+  my @components = split('/', $dir);
+  for (my $i = 0; $i < @components; $i++) {
+    my $testdir = join ('/', @components[0..$i]);
+    if (-l "$testdir") {
+      return;
+    }
+  }
+
+  # If the release file already exists, read it to see if anything has
+  # changed.  Don't recreate the file unless we're actually changing anything,
+  # to avoid unnecessary timestamp updates.
+  if (-e "$dir/Release") {
+    action(! open(REL, "$dir/Release"), "Read Release file in $dir", 2);
+    local $/;
+    my $old = <REL>;
+    close REL;
+    if ($contents ne $old) {
+      action(! open(REL, "> $dir/Release"), "Update Release file in $dir", 2);
+      print REL $contents;
+      close REL;
+    }
+  } else {
+    action(! open(REL, "> $dir/Release"), "Create Release file in $dir", 2);
+    print REL $contents;
+    close REL;
+  }
+}
+
 ###############################################################################
 ######################### EMAIL HANDLING ######################################
 ###############################################################################
@@ -640,6 +737,152 @@ sub handleScan() {
   }
 }
 
+###############################################################################
+# Name:		findSectionsArchitectures
+# Description:	Find sections and architectures in a distribution.
+# Dates:	2004-08-10      Written.
+# Arguments:    Path to distribution directory to check,
+#               reference to section array to fill out,
+#               reference to architectures array to fill out
+###############################################################################
+
+sub findSectionsArchitectures($\@\@) {
+  my ($dir, $sectionlist, $archlist) = @_;
+  my (%dirsections, %dirarches);
+  foreach $s (@sections) {
+    if (-e "$dir/$s/override") {
+      foreach $a (@architectures) {
+        $dirarches{$a} = 1;
+      }
+      $dirarches{all} = 1;
+      $dirsections{$s} = 1;
+    }
+    if (-e "$dir/$s/override.src") {
+      $dirarches{source} = 1;
+      $dirsections{$s} = 1;
+    }
+  }
+
+  # Do things this way so that the lists are in the same order as @sections
+  # and @architectures.  Purely aesthetic.
+  @$sectionlist = grep { $dirsections{$_} } @sections;
+  @$archlist = grep { $dirarches{$_} } @architectures, 'all', 'source';
+}
+
+###############################################################################
+# Name:		generateIndexConfig
+# Description:	Generate an apt-ftparchive configuration for the archive.
+# Dates:	2004-08-10      Written.
+# Uses:		%dests, $destdir, $cachedir
+# Returns:      Path to the generated config file
+###############################################################################
+
+sub generateIndexConfig() {
+  my $destcdir = $destdir;
+  $destcdir =~ s|^(.*)/[^/]+$|$1|;
+
+  my $configpath = "$destdir/.apt-ftparchive.conf";
+  action(! open(CONF, "> $configpath"), "Create apt-ftparchive config", 2);
+
+  # The common header.
+  print CONF "Dir {\n";
+  print CONF "  ArchiveDir \"$destcdir\";\n";
+  print CONF "  Cachedir \"$cachedir\";\n";
+  print CONF "};\n\n";
+  print CONF "TreeDefault {\n";
+  print CONF "  BinCacheDB \"cache.db\";\n";
+  print CONF "  Release::Origin \"$release{origin}\";\n"
+      if defined $release{origin};
+  print CONF "  Release::Label \"$release{label}\";\n"
+      if defined $release{label};
+  print CONF "};\n\n";
+
+  # The keys of %dests are all of the distribution/section/arch paths that
+  # were modified in this run.  We can have apt-ftparchive only index the
+  # distributions that were changed, but we have to reindex the entire
+  # distribution, since otherwise the Contents files won't be accurate.  Find
+  # all affected distributions from %dests, but then locate all sections and
+  # architectures under there using the handleScanAll logic.  Skip
+  # distributions that are symlinks so that we don't index the same
+  # distribution more than once.
+  my %changedist = map { s%/.*%%; $_ => 1 } keys %dests;
+  foreach $d (keys %changedist) {
+    my $codename = $distmapping{$d} || $d;
+    next if -l "$destdir/$codename";
+    print CONF "Tree \"dists/$d\" {\n";
+    my (@dsections, @darches);
+    &findSectionsArchitectures("$destdir/$d", \@dsections, \@darches);
+    print CONF "  Sections \"", join (' ', @dsections), "\";\n";
+    print CONF "  Architectures \"", join (' ', @darches), "\";\n";
+    print CONF "  Release::Suite \"$d\";\n";
+    print CONF "  Release::Codename \"$codename\";\n";
+    print CONF "};\n\n";
+  }
+  close CONF;
+  return $configpath;
+}
+
+###############################################################################
+# Name:		handleIndex
+# Description:	Handles the indexing via apt-ftparchive.
+# Dates:	2004-07-30	Written.
+#               2004-08-09      Add merging of binary-all Packages file.
+# Uses:		%dests, $destdir
+# Changes:	$ENV{PWD}
+###############################################################################
+
+sub handleIndex() {
+  $destdir =~ s|/+$||;
+
+  action(! chdir $destdir, "Change to dir $destdir", 2);
+  &destinationLock();
+
+  my $aptconfig = &generateIndexConfig();
+  cmdaction("apt-ftparchive generate $aptconfig",
+            "Index source and package files in $destdir",
+            3);
+  unlink($aptconfig);
+
+  # apt-ftparchive doesn't correctly combine binary-all/Packages with
+  # binary-arch/Packages, so we have to patch it up after the fact.
+  # apt-ftparchive reindexes the whole distribution when anything in that
+  # distribution is touched, so find modified distributions and then touch up
+  # the Packages files for each section and architecture under there.
+  #
+  # Remove binary-all/Packages after we finish with all architectures for a
+  # section, so as to not append to Packages more than once even if we revist
+  # the same area twice due to a symlink.
+  my %changedist = map { s%/.*%%; $_ => 1 } keys %dests;
+  foreach $d (keys %changedist) {
+    my (@dsections, @darches);
+    &findSectionsArchitectures("$destdir/$d", \@dsections, \@darches);
+    foreach $s (@dsections) {
+      if (-s "$d/$s/binary-all/Packages") {
+        foreach $_ (@darches) {
+          $a = $_;
+          next if ($a eq 'source' || $a eq 'all');
+          $a = "binary-$a";
+          action(! open(ARCH, ">> $d/$s/$a/Packages"),
+                 "Append to $d/$s/binary-$a/Packages",
+                 2);
+          action(! open(ALL, "$d/$s/binary-all/Packages"),
+                 "Read $d/$s/binary-all/Packages",
+                 2);
+          print ARCH <ALL>;
+          close ALL;
+          close ARCH;
+          cmdaction("gzip $d/$s/$a/Packages -c > $d/$s/$a/Packages.gz",
+                    "Compress merged Packages file",
+                    3);
+        }
+      }
+      unlink("$d/$s/binary-all/Packages", "$d/$s/binary-all/Packages.gz");
+    }
+  }
+
+  &destinationRelease();
+}
+
 ###############################################################################
 # Name:		handleSorting
 # Description:	Sort packages into the right place.
@@ -694,6 +937,7 @@ sub handleSorting() {
 # Name:		handleStructureFix
 # Description:	Fix the distribution directory structure.
 # Dates:	2001-07-23	Moved from START section to this subprocedure.
+#               2004-08-09      Call createRelease to build Release files.
 # Uses:		@distributions, @sections, @architectures, $fixstructure
 ###############################################################################
 
@@ -722,14 +966,16 @@ sub handleStructureFix() {
 	}
 	for $ar (@architectures) {
 	  createPF("$dis/$se/binary-$ar", "Packages");
+          createRelease("$dis/$se/binary-$ar", $di, $se, $ar);
 	}
 	createPF("$dis/$se/source", "Sources");
+        createRelease("$dis/$se/source", $di, $se, 'source');
 	if (! -e "$dis/$se/override") {
 	  cmdaction("touch $dis/$se/override",
 		    "Create file $dis/$se/override.",
 		    2);
 	}
-	if (! -e "$dis/$se/override") {
+	if (! -e "$dis/$se/override.src") {
 	  cmdaction("touch $dis/$se/override.src",
 		    "Create file $dis/$se/override.src.",
 		    2);
@@ -1407,84 +1653,188 @@ debarchiver [options]
 
 =head1 DESCRIPTION
 
+The debian archiver is a tool that installs debian packages into a file structure suitable for apt-get, dselect and similar tools to use for updating the installed debian system. It is ment to be used by local administrators that needs special packages, or tweaked versions to ease administration.
+
+The file structure is based on the potato file structure and does not support package pools. This may be implemented later, but is not high priority.
 
 =head1 OPTIONS
 
- --debug-level level	What information that should be printed.
-  --dl level		 1 = critical, 2 = error, 3 = normal,
-			 4 = message, 5 = debug, 6 = verbose debug (modules).
- --quit-level level	On what level to quit the application, see debug level.
- -v | --version		Prints the version string.
- --help			Prints this information.
- --copycmd		The install command to use, default $copycmd.
-			 Both packages and .changes files are installed using
-			 this command.
- --movecmd		Command to move files (currently not used at all).
- --rmcmd		The remove command to use, default $rmcmd.
-			 This can be used to move away the old packages to
-			 some other place.
- --instcmd		DEPRICATED!
- -d | --dest dir       	Destination directory. The base directory where all
-  --destdir dir		 the distribution packages reside. Here the
-			 \$distrib/\$major/\$arch/\$section directory structure
-			 will be created. Default $destdir, relative to the
-			 input directory.
- -i | --input dir	This is the directory where the all packages and
-  --indir dir		 corresponding *.changes files that should be
-  --inputdir dir	 installed to the --dest directory,
-			 default $instdir.
- --lockfile file	The lockfile to use, default $lockfile.
- --cinstall dir		Where the .changes file will be installed to,
-			 empty string to remove it instead, default $cinstall.
- --distinputcritera     The criteria for which binary packages that should be
-			 installed even if it does not have a .changes file,
-			 default $distinputcriteria.
- -o | --addoverride    	Automaticly add new packages to the override file.
- --autoscanpackages	Automaticly run dpkg-scanpackages after all new
-			 packages are installed.
- --autoscansources	Automaticly run dpkg-scansources after all new
-			 packages are installed.
- -a | --autoscan       	Does both --autoscanpackages and --autoscansources.
- --scanall		Scan all distributions, sections, etc.
- --autoscanall		Same as --scanall --autoscan.
- --nosort		Do not sort packages.
- --nostructurefix	Do not create directories and touch Package files.
- --scanonly		Same as --nosort --nostructurefix.
+=over 4
+
+=item B<--debug-level | --dl> level
+
+What information that should be printed. 1=critical, 2=error, 3=normal, 4=message, 5=debug, 6=verbose debug (modules).
+
+=item B<--quit-level> level
+
+On what level to quit the application, see debug level.
+
+=item B<-v | --version>
+
+Prints the version string.
+
+=item B<--help>
+
+Prints this information.
+
+=item B<--copycmd>
+
+The install command to use, default $copycmd. Both packages and .changes files are installed using this command.
+ 
+=item B<--movecmd>
+
+Command to move files (currently not used at all).
+
+=item B<--rmcmd>
+
+The remove command to use, default $rmcmd. This can be used to move away the old packages to some other place.
+
+=item B<--instcmd>
+
+DEPRICATED!
+
+=item B<-d | --dest | --destdir> dir
+
+Destination directory. The base directory where all the distribution packages reside. Here the $distrib/$major/$arch/$section directory structure will be created. Default $destdir, relative to the input directory.
+
+=item B<--scandetect | -s>
+
+Scan using apt-ftparchive or dpkg-scan* depending on what is installed on the system. This is the recommended way. Only use --index or --autoscan if you know what you are doing.
+ 
+=item B<-x | --index>
+
+Automatically run apt-ftparchive after all new packages are installed.  config must be an absolute path to the configuration file to use for apt-ftparchive generate. See the apt-ftparchive manual page for more information. Use this *or* --autoscan, not both.
+
+=item B<-i | --input | --indir | --inputdir> dir
+
+This is the directory where the all packages and corresponding *.changes files that should be installed to the --dest directory, default $instdir.
+
+=item B<--cachedir> dir
+
+The apt-ftparchive package cache directory, if --index is used.  Default $cachedir.
+
+=item B<--lockfile> file
+
+The lockfile to use, default $lockfile.
+
+=item B<--cinstall> dir
+
+Where the .changes file will be installed to, empty string to remove it instead, default $cinstall.
+
+=item B<--distinputcriteria>
+
+The criteria for which binary packages that should be installed even if it does not have a .changes file, default $distinputcriteria.
+ 
+=item B<-o | --addoverride>
+
+Automaticly add new packages to the override file.
+
+=item B<--autoscanpackages>
+
+Automaticly run dpkg-scanpackages after all new packages are installed.
+
+=item B<--autoscansources>
+
+Automaticly run dpkg-scansources after all new packages are installed.
+
+=item B<-a | --autoscan>
+
+Does both --autoscanpackages and --autoscansources.
+
+=item B<--scanall>
+
+Scan all distributions, sections, etc.
+
+=item B<--autoscanall>
+
+Same as --scanall --autoscan.
+
+=item B<--nosort>
+
+Do not sort packages.
+
+=item B<--nostructurefix>
+
+Do not create directories and touch Package files.
+
+=item B<--scanonly>
+
+Same as --nosort --nostructurefix.
 
 =head1 CONFIG FILE
 
-You can also place config files with the following names (in following order)
-/etc/debarchiver.conf, ~/.debarchiver.conf and input.conf (relative to input
-directory) that will be read before, the arguments to this program
-will be parsed. Here you can change the following variables:
+You can also place config files with the following names (in following order) /etc/debarchiver.conf, ~/.debarchiver.conf and input.conf (relative to input directory) that will be read before, the arguments to this program will be parsed. Here you can change the following variables:
 
-	$destdir      	The destination directory (see --destdir above).
-	$inputdir	The input directory (no effect in $inputconfigfile).
-	$copycmd	The install command (see --copycmd).
-	$movecmd	The move command (see --movecmd).
-	$rmcmd		The remove command (see --rmcmd above).
-	$cinstall	Where the .changes files are installed
-			 (see --cinstall above).
-        $distinputcritera The criteria for which packages that should be
-			 installed even if it does not have a .changes file,
-			 default $distinputcriteria.
-        %distinputdirs The directories (distribution => dir) that should be
-			 searched for extra bianry packages that does not need
-			a .changes file to be installed.
-	$lockfile	The lockfile to use, default $lockfile.
-	@mailtos	An array of strings that should be mailed to.
-			If the string contains en email address that one is
-			used. If it contains an incomplete email address, i.e.
-    			@hostname, the username owning the file is used @
-    			the hostname specified. If no @ character is found in
-    			the string, it is considered as a field in the
-    			.changes file. Such a field can for example be
-    			Maintainer or Uploaders.
+=item B<$destdir>
+
+The destination directory (see --destdir above).
+
+=item B<$inputdir>
+
+The input directory (no effect in $inputconfigfile).
+
+=item B<$cachedir>
+
+The cache directory for apt-ftparchive, used if --index is used.
+
+=item B<$copycmd>
+
+The install command (see --copycmd).
+
+=item B<$movecmd>
+
+The move command (see --movecmd).
+
+=item B<$rmcmd>
+
+The remove command (see --rmcmd above).
+
+=item B<$cinstall>
+
+Where the .changes files are installed (see --cinstall above).
+
+=item B<$distinputcriteria>
+
+The criteria for which packages that should be installed even if it does not have a .changes file, default $distinputcriteria.
+
+=item B<%distinputdirs>
+
+The directories (distribution => dir) that should be searched for extra bianry packages that does not need a .changes file to be installed.
+
+=item B<$lockfile>
+
+The lockfile to use, default $lockfile.
+
+=item B<@mailtos>
+
+An array of strings that should be mailed to. If the string contains en email address that one is used. If it contains an incomplete email address, i.e. @hostname, the username owning the file is used @ the hostname specified. If no @ character is found in the string, it is considered as a field in the.changes file. Such a field can for example be Maintainer or Uploaders.
+
+=item B<%release>
+
+Additional information to add to generated Release files.  Supported keys are origin, label, and description.
+
+=head1 PACKAGE INDEXING
+
+There are two ways to generate the indexes that B<apt-get> relies on.
+
+Using B<--autoscanpackages>, B<--autoscansources>, or B<--autoscan> will use B<dpkg-scanpackages> and B<dpkg-scansources>.  This will generate the Packages and Sources files, but will not generate Contents files and can be slow with a large repository.
+
+Alternately, the B<--index> I<config> option will call B<apt-ftparchive> to index the package tree.  B<apt-ftparchive> can also generate Contents files (for use with B<apt-file>), and can optionally use a cache of package information to speed up multiple runs.  The B<apt-ftparchive> configuration file will be generated automatically. This is however not fully tested.
+
+You should use either B<--autoscanpackages> and B<--autoscansources> or B<--index>, not both, as they both do basically the same thing.
+
+The default action (and the recommended) is B<--scandetect> that probe for installed software and use the best choice depending on what software you have installed (choose between --index and --autoscan right now).
+
+=head1 FILES
+
+B</etc/debarchiver.conf>
+
+=head1 SEE ALSO
+
+B<apt-ftparchive>(1)
 
 =head1 AUTHOR
 
 Ola Lundqvist <opal@debian.org>
 
-=head1 SEE ALSO
-
 =cut
